/*
 * MIDIRECT.H -- LIBMI direct access interface
 *			and unified server/registered function calling
 *
 *     Copyright c 1995 Illustra Information Technologies, Inc.
 *     All rights reserved.      See files/copyright.
 *
 */

#ifndef _MIDIRECT_H_
#define _MIDIRECT_H_

#ifndef _MILIB_H_
#include "milib.h"
#endif

typedef struct mi_table_desc	MI_TABLE_DESC;
typedef struct mi_scan_desc	MI_SCAN_DESC;
typedef struct mi_value_info	MI_VALUE_INFO;

typedef struct mi_funcdesc	MI_FUNCDESC;

/* flags for direct function calling */
#define MI_NOFPARAM		0x00000010	/* no fparam */
#define MI_USERFPARAM		0x00000020	/* user supplies fparam */
#define MI_ALLOCFPARAM		0x00000040	/* lib creates fparam */

#if defined(WINNT) || defined(MI_WIN31_PORT)
EXTERNC_BEGIN
MI_FUNCDESC	MI_FAR * MI_FAR * MI_PROC_EXPORT GetMiCurFd ARGS((void ));
EXTERNC_END
#define _mi_cur_fd	(*(GetMiCurFd()))
#else
extern MI_FUNCDESC	MI_FAR *_mi_cur_fd;
#endif

#define MI_EXEC_FUNCTION(conn, sig, arglist) \
    (_mi_cur_fd = (mi_get_function((conn), (sig), MI_NOFPARAM)), \
     (MI_FD_ERROR(_mi_cur_fd) \
	? ((char MI_FAR *) NULL) \
	: (mi_funcdesc_push(_mi_cur_fd), \
	   mi_funcdesc_pop_and_ret((*(_mi_cur_fd->fd_funcptr)) arglist))))

#define MI_EXEC_FUNCTION_FPARAM(conn, sig, arglist) \
    (_mi_cur_fd = mi_get_function((conn), (sig), MI_USESFPARAM), \
     (MI_FD_ERROR(_mi_cur_fd) \
	? ((char MI_FAR *) NULL) \
	: (mi_funcdesc_push(_mi_cur_fd), \
	   mi_funcdesc_pop_and_ret((*(_mi_cur_fd->fd_funcptr)) arglist))))

#define MI_CALL_FUNCTION(fd, arglist) \
	(_mi_cur_fd = (fd), (*(_mi_cur_fd->fd_funcptr)) arglist)
	

/*--------------------------
 * Public function prototypes.
 *--------------------------
 */

EXTERNC_BEGIN

void * MI_PROC_EXPORT
mi_call_function ARGS((void *first, ...));

mi_integer MI_PROC_EXPORT
mi_command_counter_increment ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_exec_function_by_id ARGS((MI_CONNECTION MI_FAR *conn,
			     const char MI_FAR *func_id,
			     mi_integer nargs,
			     const mi_integer MI_FAR *arg_lengths,
			     void MI_FAR * MI_FAR *arguments,
			     mi_integer retlen,
			     mi_integer retinfo));

mi_integer MI_PROC_EXPORT
mi_exec_function_fparam ARGS((MI_CONNECTION MI_FAR *conn,
			      const char MI_FAR *func_id,
			      mi_integer nargs,
			      const mi_integer MI_FAR *arg_lengths,
			      void MI_FAR * MI_FAR *arguments,
			      mi_integer retlen,
			      mi_integer retinfo,
			      MI_FPARAM MI_FAR *fparam));

void MI_PROC_EXPORT
mi_funcdesc_free ARGS((MI_FUNCDESC MI_FAR *fdesc));

void MI_PROC_EXPORT
mi_funcdesc_push ARGS((MI_FUNCDESC MI_FAR *fdesc));

char MI_FAR * MI_PROC_EXPORT
mi_funcdesc_pop_and_ret ARGS((char MI_FAR *funcval));

MI_FUNCDESC MI_FAR * MI_PROC_EXPORT
mi_funcdesc_top ARGS((void));

mi_integer MI_PROC_EXPORT
mi_function_return_value ARGS((MI_CONNECTION MI_FAR *conn,
			       void MI_FAR * MI_FAR *retval,
			       mi_integer MI_FAR *retlen,
			       mi_integer MI_FAR *resultstate));

MI_FUNCDESC MI_FAR * MI_PROC_EXPORT
mi_get_function ARGS((MI_CONNECTION MI_FAR *conn,
		      const MI_STRING MI_FAR *sig,
		      mi_integer flags));

MI_ROW_DESC MI_FAR * MI_PROC_EXPORT
mi_get_table_row_desc ARGS((MI_TABLE_DESC MI_FAR *table));

mi_integer MI_PROC_EXPORT
mi_row_object_id ARGS((MI_ROW MI_FAR *row,
		       char MI_FAR *buffer,
		       mi_integer buflen));

mi_integer MI_PROC_EXPORT
mi_row_table_offset ARGS((MI_ROW MI_FAR *row,
			  char MI_FAR *buffer,
			  mi_integer buflen));

MI_SCAN_DESC MI_FAR * MI_PROC_EXPORT
mi_scan_begin ARGS((MI_TABLE_DESC MI_FAR *table,
		    const MI_STRING MI_FAR *indname,
		    mi_integer n_indexkeys,
		    mi_integer n_otherkeys,
		    MI_STRING MI_FAR * MI_FAR *indexkey_cmps,
		    MI_STRING MI_FAR * MI_FAR *otherkey_cmps,
		    mi_integer MI_FAR *keynums,
		    mi_integer MI_FAR *columns,
		    MI_VALUE_INFO MI_FAR *indexkey_values,
		    MI_VALUE_INFO MI_FAR *otherkey_values));

mi_integer MI_PROC_EXPORT
mi_scan_end ARGS((MI_SCAN_DESC MI_FAR *scan));

mi_integer MI_PROC_EXPORT
mi_scan_mark_position ARGS((MI_SCAN_DESC MI_FAR *scan));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_scan_next_row ARGS((MI_SCAN_DESC MI_FAR *scan,
		       mi_integer forward,
		       mi_integer binary,
		       mi_integer MI_FAR *error));

mi_integer MI_PROC_EXPORT
mi_scan_restore_position ARGS((MI_SCAN_DESC MI_FAR *scan));

mi_integer MI_PROC_EXPORT
mi_table_close ARGS((MI_TABLE_DESC MI_FAR *table));

MI_TABLE_DESC MI_FAR * MI_PROC_EXPORT
mi_table_open ARGS((MI_CONNECTION MI_FAR *conn,
		    const MI_STRING MI_FAR *table_name,
		    const MI_STRING MI_FAR *starttime,
		    const MI_STRING MI_FAR *endtime));

mi_integer MI_PROC_EXPORT
mi_table_row_delete ARGS((MI_TABLE_DESC MI_FAR *table,
			  const char MI_FAR *offset));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_table_row_fetch ARGS((MI_TABLE_DESC MI_FAR *table,
			 const char MI_FAR *offset,
			 mi_integer binary,
			 mi_integer MI_FAR *error));

char MI_FAR * MI_PROC_EXPORT
mi_table_row_insert ARGS((MI_TABLE_DESC MI_FAR *table,
			  MI_VALUE_INFO MI_FAR *values));

char MI_FAR * MI_PROC_EXPORT
mi_table_row_update ARGS((MI_TABLE_DESC MI_FAR *table,
			  const char MI_FAR *offset,
			  mi_integer ncolumns,
			  mi_integer MI_FAR *columns,
			  MI_VALUE_INFO MI_FAR *values));

MI_VALUE_INFO MI_FAR * MI_PROC_EXPORT
mi_value_info_create ARGS((mi_integer nvalues));

MI_VALUE_INFO MI_FAR * MI_PROC_EXPORT
mi_value_info_create_and_set ARGS((mi_integer nvalues,
				   void MI_FAR * MI_FAR *values,
				   const mi_integer MI_FAR *lengths,
				   const mi_integer MI_FAR *info,
				   mi_integer binary));

mi_integer MI_PROC_EXPORT
mi_value_info_destroy ARGS((MI_VALUE_INFO MI_FAR *infostruct));

mi_integer MI_PROC_EXPORT
mi_value_info_set ARGS((MI_VALUE_INFO MI_FAR *infostruct,
			mi_integer entry,
			const void MI_FAR *value,
			mi_integer length,
			mi_integer info,
			mi_integer binary,
			mi_integer force));
EXTERNC_END

#endif /* _MIDIRECT_H_ */

